# sb::Admin::Login
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Login;
use strict;
use sb::App::Admin ();
use base qw( sb::App::Admin );
use vars qw( $VERSION );
$VERSION = '0.08';
# 0.08 [2009/03/19] removed br tags from the strings
# 0.07 [2008/08/30] modified _display to ignore permission warning temporarily
# 0.06 [2007/12/03] added _check_init_status to warn init_file status
# 0.05 [2007/09/27] changed sb_body_class from "main" to "login"
# 0.04 [2007/09/15] changed _display to consider invalid mode
# 0.03 [2007/09/08] ported from Serene Bach 2.*
# 0.02 [2005/08/11] changed _open_login_panel to put hidden parameters correctly
# 0.01 [2005/06/07] changed _open_login_panel to implement bookmarklet
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE    (){ 'main.html' };
sub COOKIE_USER (){ 'user' };
my @mIgnoreParam = ('__user','__pass','upload_file','temp_package','import_data');
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	return $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $bookmarklet = ();
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $cookie = $self->cgi->cookie('name'=>sb::Config->get->value('basic_admntag') . COOKIE_USER);
	# --- main template parts instead of calling set_main
	$self->common_parts($temp);
	$self->_set_title($temp);
	$temp->num(0);
	$temp->tag('sb_body_class' => ($self->cgi->value('__mode') ne 'bm') ? 'login' : 'bm');
	$temp->block('sb_mainbody' => 0);
	$temp->block('sb_login' => 1);
	$temp->block('sb_mainhead' => 1) if ($self->cgi->value('__mode') ne 'bm');
	# --- rest of parts
	$temp->tag('cookie_user'=>( $self->{'user'} ) ? $self->{'user'}->name : $cookie->{'user'} );
	$temp->tag('sb_head_help_url'=>$self->conf->value('srv_doc') . $self->lang->language . '.html');
	if ( $param{'message'} ne sb->string('Not authorized.') )
	{
		my @modes = map { $_->[0] } @{$self->list};
		my $hidden = '';
		foreach my $key ( $self->cgi->names )
		{
			my $value = $self->cgi->value($key);
			next if (grep(/^$key/,@mIgnoreParam));
			next if ($key eq '__mode' and !grep(/^\Q$value\E$/,@modes));
			$hidden .= '<input type="hidden" name="' . $key . '" value="' . sb::Text->new($value)->entitized() . '" />';
		}
		$temp->tag('sb_bm_para'=>$hidden);
	}
	# $param{'message'} = $self->_check_init_status($param{'message'});
	$self->set_message($temp,$param{'message'});
	return $temp->output;
}
sub _check_init_status
{
	my $self = shift;
	my $msg  = shift;
	if ( $self->conf->value('basic_warn_init')
	 and -w $self->conf->value('init_file'))
	{
		my $string = ($msg eq '') ? '' : "\n";
		$string .= sb->string('%s is writable.  Disabling writable permission is recommended.');
		$msg .= sprintf($string,$self->conf->value('init_file'));
	}
	return $msg;
}
1;
__END__
