# sb::App::Builder
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Builder;
use strict;
use sb::Data ();
use sb::Time ();
use sb::Lock ();
use sb::Session ();
use sb::Build ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.08';
# 0.08 [2009/02/19] modified _build_files to run callback
# 0.07 [2009/02/17] modified _build_files to use new API of sb::Build
# 0.06 [2007/08/16] modified for Serene Bach 3.*
# 0.05 [2005/08/23] changed _build_files to build correctly when number of article is below max number of building
# 0.04 [2005/08/23] changed _check_mode to check session correctly
# 0.03 [2005/08/05] chnaged _build_files to build all files correctly
# 0.02 [2005/07/24] changed _build_files to check whether xml request is acceptable or not
# 0.01 [2005/07/22] changed _build_file to allow to build multiple files at once
# 0.00 [2005/07/18] generated
# --------------------------------------
# // defines / static variables
sub COOKIE_USER     (){ 'user' };
sub CONTENT_TYPE    (){ 'text/xml' };
sub OUTPUT_CHARSET  (){ 'utf-8' };
sub OUTPUT_TEMPLATE (){ "<?xml version=\"1.0\"?>\n<result>%s</result>\n" };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(@_);
}
sub run
{
	my $class = shift;
	my $self  = $class->new(@_);
	my $error = $self->_check_mode();
	if (!$error)
	{
		my $lock = sb::Lock->lock;
		if ($lock)
		{
			$error = $self->_build_files(int($self->cgi->value('__rebuild')));
			$lock->unlock();
		}
		else
		{
			$error = 'Failed to lock';
		}
	}
	my $output = sprintf(OUTPUT_TEMPLATE,$error);
	$self->cgi->head(
		'type'    => CONTENT_TYPE,
		'charset' => OUTPUT_CHARSET,
		'length'  => length($output),
	);
	$self->output($output);
}
# --------------------------------------
# // private functions
sub _check_mode
{ # checking session and user
	my $self = shift;
	my $cookie = $self->cgi->cookie('name'=>$self->conf->value('basic_admntag') . COOKIE_USER);
	return 'wrong user' if ($cookie->{'user'} eq '');
	my $user = sb::Data->load('User','cond'=>{'name'=>$cookie->{'user'}},'num'=>1);
	return 'wrong user' if (!$user);
	my $session = sb::Session->new(
		'time'   => $self->time->data(),
		'key'    => $user->id,
		'path'   => $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_admn'),
		'name'   => $self->conf->value('basic_sessiontag'),
		'expire' => $self->conf->value('basic_admn_expire'),
	);
	return 'session expires' if ( !$session->check );
	return;
}
sub _build_files
{ # rebuild files
	my $self = shift;
	my $num  = shift;
	my $output = undef;
	my %cats = sb::Data->load_as_hash('Category');
	my $builder = sb::Build->new(
		'time'      => $self->time->data(),
		'user'      => { sb::Data->load_as_hash('User') },
		'cat'       => \%cats,
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%cats) ],
		'blog'      => sb::Data->load('Weblog','id'=>0),
		''
	);
	$builder->run_callback('prebuild_all') if ($num == 0);
	$builder->entryinfo();
	if ($num == 0)
	{ # --- it's the first build process
		$builder->build_indexes();
		$output = ($self->conf->value('conf_entry_archive') eq 'None') ? 'completed' : 1;
	}
	elsif ($num > 0)
	{ # --- articles
		my ($now,$nxt) = $builder->build_ranged_entries($num - 1,$self->conf->value('basic_build_ajax'));
		$output = 
			  (!$now) ? 'no entry'
			: (!$nxt) ? 'completed'
			:           1;
	}
	else
	{ # $num < 0
		$output = 1;
	}
	$builder->run_callback('postbuild_all') if ($output eq 'completed');
	return $output;
}
1;
__END__
