# sb::Build
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Build;
use strict;
use sb::Language ();
use sb::Config ();
use sb::Data ();
use sb::Content ();
use sb::Content::Feed ();
use sb::Template ();
use sb::App ();
use sb::App::Feed ();
use sb::Utils ();
use sb::Plugin ();
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.19';
# 0.19 [2009/03/21] added _set_script_tags to modify tags to use js instead of original output
# 0.18 [2009/02/27] modified build_category_index / build_monthly_archive to use js lists
# 0.17 [2009/02/19] added 'callback' as a member variable / added register_js_list, _run_callback
# 0.16 [2009/02/18] modified build_css to check file existence
# 0.15 [2009/02/17] added build_indexes/build_ranged_entries/construct_entry/build_entry_on_receipt
# 0.14 [2009/01/14] modified build_category_index to set correct category id for checking template
# 0.13 [2007/11/30] modified initialize to use sb instaed of different classes
# 0.12 [2007/09/20] changed _check_template to use "template" method instead of $self->{'template'}
# 0.11 [2007/08/17] removed some setter functions no longer used
# 0.10 [2007/08/17] changed set_callback_for_feed and build_feedfile to use sb::Content::Feed
# 0.09 [2007/08/12] modified for Serene Bach 3.*
# 0.08 [2007/05/11] changed build_category_index and build_monthly_archive
# 0.07 [2006/02/03] changed build_feedfile
# 0.06 [2005/10/18] changed _initialize to load templates as instances of TemplateManager
# 0.05 [2005/08/03] changed build_top_page to sort entries correctly
# 0.04 [2005/08/02] changed set_latest_entries to get entries for feeds
# 0.03 [2005/07/16] changed build_javascript, _script_tag, _generate_javascript to handle list type block correctly
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/07/08] added build_cookie_js
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines / static variables
sub SCRIPT_TAG         (){ '<script type="text/javascript" charset="%s" src="%s"></script>' };
sub SUBCATEGORY_PREFIX (){ 'cat' };
sub LIST_TEMPLATE      (){ "<!-- BEGIN %s -->\n{%s}\n<!-- END %s -->\n" };
sub COOKIE_JS_TEMP     (){ 'default_script.js' };
sub JS_FILE  (){ 0 };
sub JS_BLOCK (){ 1 };
sub JS_ENTRY (){ 2 };
my %mListScripts = (
	# 'tag_name'            => ['*.js',      'block',           'ent',],
	'link_list'             => ['link.js',   'link',            undef,],
	'user_list'             => ['user.js',   'profile',         undef,],
	'recent_comment_list'   => ['comment.js','recent_comment',  undef,],
	'recent_trackback_list' => ['tb.js',     'recent_trackback',undef,],
	'latest_entry_list'     => ['entry.js',  'latest_entry',    1,],
	'category_list'         => ['cat.js',    'category',        1,],
	'archives_list'         => ['arc.js',    'archives',        1,],
	'calendar'              => ['cal1.js',   'calendar',        1,],
	'calendar2'             => ['cal2.js',   'calendar',        1,],
	'calendar_vertical'     => ['calv.js',   'calendar',        1,],
	'calendar_horizontal'   => ['calh.js',   'calendar',        1,],
);
my %mFeedFile = (
	'rss'  => 'index.rdf',
	'atom' => 'atom.xml',
);
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'time'       => undef, # [NUM.]current time
		'blog'       => undef, # [OBJ.]weblog data
		'user'       => undef, # [HASH]user data
		'cat'        => undef, # [HASH]category data
		'sortedcat'  => undef, # [ARRY]sorted category data
		'entry'      => undef, # [ARRY]the latest entries
		'entry_num'  => undef, # [NUM.]number of entries
		'entryinfo'  => undef, # [HASH]entry information
		'template'   => undef, # [HASH]template data
		'default_id' => undef, # [NUM.]default template id
		'js_list'    => {},    # [HASH]javascript files
		'callback'   => [],    # [ARRY]callbacks
		'depth'      => 0,     # [NUM.]iteration checker
		'conf' => sb->get->conf,
		'lang' => sb->get->lang,
	);
	$param{'js_list'} = \%mListScripts;
	$param{'callback'} = [sb::Plugin->load_extra_module('build')];
	$self->SUPER::initialize(%param,@_);
	$self->generate_accessor('time','blog','user','cat','sortedcat','entry','conf','lang');
	$self->run_callback('initialize');
	return $self;
}
# --------------------------------------
# // public functions - executor
sub construct_entry
{
	my $self = shift;
	my $target = shift;   # the target entry to be constructed
	my $neighbor = shift; # [array ref] the neighbor entries of the target
	my $oldstate = shift; # [hash ref] previous state of the target (optional)
	my $type = $self->{'conf'}->value('conf_entry_archive');
	return if ($self->{'depth'} > 0);
	$self->{'depth'}++;
	$self->run_callback('prebuild_entry',$target);
	if ($type eq 'Individual')
	{
		my @entries = ($target);
		my @ids = ($target->id);
		if ( $oldstate )
		{
			push(@ids,$oldstate->{'prev'}->id) if ($oldstate->{'prev'});
			push(@ids,$oldstate->{'next'}->id) if ($oldstate->{'next'});
		}
		push(@ids,$neighbor->[0]->id) if ($neighbor->[0]);
		push(@ids,$neighbor->[1]->id) if ($neighbor->[1]);
		@ids = sb::Utils->remove_duplicates(@ids);
		foreach my $id ( @ids )
		{
			next if ($id == $target->id);
			my $obj = sb::Data->load('Entry','id'=>$id);
			push(@entries,$obj) if ($obj);
		}
		foreach ( keys %{$self->{'js_list'}} )
		{
			next if (!$self->{'js_list'}->{$_}->[JS_ENTRY]);
			$self->build_javascript( $_ );
		}
		foreach ( @entries )
		{
			next if ($_->stat == 0);
			$self->build_entry( $_ );
		}
	}
	elsif ($type eq 'Monthly')
	{
		my $month = sb::Time->format(
			'time'=>$target->date,
			'form'=>'%Year%%Mon%',
			'zone'=>$target->tz
		);
		$self->build_monthly_archive( $month );
		if ( $oldstate )
		{
			my $check = sb::Time->format(
				'time'=>$oldstate->{'date'},
				'form'=>'%Year%%Mon%',
				'zone'=>$oldstate->{'zone'}
			);
			$self->build_monthly_archive( $check ) if ($check ne $month);
		}
	}
	$self->run_callback('postbuild_entry',$target);
	my $categories = join(',',$target->cat,$target->add);
	foreach my $cid ( split(',',$categories) )
	{
		next if ($cid eq '');
		next if (!$self->{'cat'}->{$cid});
		$self->build_category_index( $cid );
	}
	$self->set_latest_entries();
	$self->build_top_page();
	$self->build_feedfile('all');
	$self->build_css(undef,'overwrite'=>0);
	$self->build_cookie_js();
	$self->{'depth'}--;
}
sub build_entry_on_receipt
{
	my $self = shift;
	my $entry = shift;
	my $mode  = shift;
	my $type  = $self->{'conf'}->value('conf_entry_archive');
	return if ($self->{'depth'} > 0);
	$self->{'depth'}++;
	$self->run_callback('prebuild_receipt',$entry);
	if ($type eq 'Individual')
	{
		$self->build_javascript('recent_comment_list') if ($mode eq 'com');
		$self->build_javascript('recent_trackback_list') if ($mode eq 'tb');
		$self->build_entry( $entry ) if ($entry->stat != 0);
	}
	elsif ($type eq 'Monthly')
	{
		my $month = sb::Time->format(
			'time'=>$entry->date,
			'form'=>'%Year%%Mon%',
			'zone'=>$entry->tz
		);
		$self->build_monthly_archive( $month );
	}
	$self->run_callback('postbuild_receipt',$entry);
	$self->build_category_index( $entry->cat ) if ($entry->cat ne '');
	$self->build_top_page();
	$self->{'depth'}--;
}
sub build_indexes
{
	my $self = shift;
	my $type = $self->{'conf'}->value('conf_entry_archive');
	return if ($self->{'depth'} > 0);
	$self->{'depth'}++;
	foreach my $cat ( values(%{$self->{'cat'}}) )
	{ # category indexes
		next if (!$cat->idx);
		next if ($cat->dir eq '');
		$self->build_category_index($cat->id);
	}
	$self->set_latest_entries();
	$self->build_top_page();
	$self->build_feedfile('all');
	$self->build_javascript('all') if ($type ne 'None');
	$self->build_css();
	$self->build_cookie_js('force_to_create');
	$self->{'depth'}--;
}
sub build_ranged_entries
{
	my $self = shift;
	my $num  = shift; # offset of entries, -1 means all entries
	my $max  = shift; # number of entries to be built
	my $type = $self->{'conf'}->value('conf_entry_archive');
	my @entries;
	return if ($self->{'depth'} > 0);
	$self->{'depth'}++;
	if ($type eq 'Individual')
	{
		@entries = sb::Data->load('Entry',
			'cond'   => {'stat'=>[1,2]}, # picking opened entries up
			'sort'   => 'date',
			'order'  => 1,
			'num'    => ($num > -1) ? $max + 2 : -1,
			'bgn'    => ($num >  0) ? ($num * $max - 1) : 0,
			'detail' => 'on',
		);
		my $bgn = ($num > 0) ? 1 : 0;
		my $end = @entries;
		if ($num > -1 and $end > $max)
		{
			$end = $max if ($num == 0);
			$end = $max + 1 if ($end > $max + 1);
		}
		for (my $i=$bgn;$i<$end;$i++)
		{
			my $nxt_ent = ($i > 0)         ? $entries[$i - 1] : undef;
			my $prv_ent = ($i < $#entries) ? $entries[$i + 1] : undef;
			$self->build_entry($entries[$i],'prev'=>$prv_ent,'next'=>$nxt_ent);
		}
		$self->{'depth'}--;
		return ( $entries[$end - 1], $entries[$end] );
	} # end of if ($type eq 'Individual')
	elsif ($type eq 'Monthly')
	{
		@entries = sb::Data->load('Entry',
			'cond'  => {'stat'=>[1,2]},
			'sort'  => 'date',
			'order' => 1,
		);
		my %check = ();
		for (my $i=0;$i<@entries;$i++)
		{
			my $month = sb::Time->format(
				'time' => $entries[$i]->date,
				'form' => '%Year%%Mon%',
				'zone' => $entries[$i]->tz
			);
			$check{$month} = $month if ( !defined($check{$month}) );
		}
		my @months = sort { $b <=> $a } keys(%check);
		if ( $num > -1 )
		{
			if ( $months[ $num ] )
			{
				$self->build_monthly_archive($months[ $num ]);
			}
			$self->{'depth'}--;
			return ($months[ $num ], $months[ $num + 1 ]);
		}
		# $num == -1, so we build all monthly archives
		foreach ( @months )
		{
			$self->build_monthly_archive( $_ );
		}
	} # end of if ($type eq 'Monthly')
	$self->{'depth'}--;
	return;
}
# --------------------------------------
# // public functions - individual executor
sub build_css
{
	my $self = shift;
	my $css  = shift; # an instance of sb::Template for css
	my %param = (
		'file' => '', # output file path if necessary
		'overwrite' => 1, # overwrite option
		@_
	);
	my $file = $param{'file'}; 
	$css  = $self->template()->{'css'} if (!$css);
	$file = $self->{'conf'}->value('conf_dir_base') . $self->{'conf'}->value('file_css') if ($file eq '');
	return if ( -s $file and !$param{'overwrite'} );
	eval {
		open(CSSOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(CSSOUT);
		print CSSOUT sb::Content->output($css,'mode'=>'css');
		close(CSSOUT);
	};
	return $@ if ($@);
	return;
}
sub build_entry
{
	my $self = shift;
	my $entry = shift;
	my %param = (
		'prev' => undef,
		'next' => undef,
		@_
	);
	return sb->string('No article.') if (!$entry);
	my $content = sb::Content->new(
		'main' => { $self->_set_script_tags('entry') }
	);
	my $file = $entry->file_path($self->{'cat'});
	eval {
		my $cat_id = ($entry->cat == -1) ? '' : $entry->cat;
		my ($base,$css) = $self->_check_template('ent',$cat_id);
		my $info = (defined($param{'prev'}) or defined($param{'next'}))
		         ? {'neighbor'=>{'prev'=>$param{'prev'},'next'=>$param{'next'}}}
		         : undef;
		open(ENTRYOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(ENTRYOUT);
		print ENTRYOUT $content->output($base,
			'mode'      => 'ent',
			'css'       => $css,
			'id'        => $entry->id,
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
			'entryinfo' => $info,
			'entry'     => [$entry],
			'entry_num' => 1,
		);
		close(ENTRYOUT);
		chmod($self->{'conf'}->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_monthly_archive
{
	my $self = shift;
	my $month = shift;
	return sb->string('Unknown error.') if (!$month);
	my $file = $self->{'conf'}->value('conf_dir_base') 
	         . $self->{'conf'}->value('conf_dir_log')
	         . $month
	         . $self->{'conf'}->value('basic_suffix');
	my $content = sb::Content->new(
		'main' => { $self->_set_script_tags('monthly') }
	);
	eval {
		my ($base,$css) = $self->_check_template('arc',undef);
		open(MONTHLYOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(MONTHLYOUT);
		print MONTHLYOUT $content->output($base,
			'mode'      => 'arc',
			'css'       => $css,
			'cond'      => $month,
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
		);
		close(MONTHLYOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_category_index
{
	my $self = shift;
	my $cat_id = shift;
	return sb->string('Unknown error.') if ($cat_id eq '' or $cat_id == -1);
	my $cat = $self->{'cat'}->{$cat_id};
	return sb->string('Unknown error.') if (!$cat);
	return if ($cat->dir eq '' or !$cat->idx); # it's not necessary to create the index.
	my $file = $self->{'conf'}->value('conf_dir_base') . $cat->dir . $self->{'conf'}->value('file_index');
	my $content = sb::Content->new(
		'main' => {
			'_rssfeed' => \&_category_feeds,
			$self->_set_script_tags('category')
		}
	);
	eval {
		my ($base,$css) = $self->_check_template('arc',$cat->id);
		open(CATOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(CATOUT);
		print CATOUT $content->output($base,
			'mode'      => 'cat',
			'id'        => $cat->id,
			'css'       => $css,
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
			'entryinfo' => $self->entryinfo(),
		);
		close(CATOUT);
		chmod($self->{'conf'}->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_top_page
{
	my $self = shift;
	my $file = $self->{'conf'}->value('conf_dir_base') . $self->{'conf'}->value('file_index');
	$self->run_callback('prebuild_top');
	my $content = sb::Content->new();
	eval {
		my ($base,$css) = $self->_check_template(undef,undef);
		open(TOPOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(TOPOUT);
		print TOPOUT $content->output($base,
			'mode'      => 'page',
			'css'       => $css,
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
			'entryinfo' => $self->entryinfo(),
			'entry'     => ( $self->{'conf'}->value('conf_entry_sort') ) ? $self->{'entry'} : undef,
			'entry_num' => $self->{'entry_num'},
		);
		close(TOPOUT);
		chmod($self->{'conf'}->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	$self->run_callback('postbuild_top');
	return;
}
sub build_javascript
{
	my $self = shift;
	my $type = shift;
	my $dir = $self->{'conf'}->value('conf_dir_base') . $self->{'conf'}->value('conf_dir_log');
	my @types = keys( %{$self->{'js_list'}} );
	if (grep { /\Q$type\E/ } @types)
	{
		@types = ($type);
	}
	else
	{
		$type = 'all';
	}
	my $content = sb::Content->new();
	foreach my $type ( @types )
	{
		my $block = $self->{'js_list'}->{$type}->[JS_BLOCK];
		my $file = $dir . $self->{'js_list'}->{$type}->[JS_FILE];
		my $cms  = sb::Template->new(sprintf(LIST_TEMPLATE,$block,$type,$block));
		my $temp = $content->output($cms,
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
			'entryinfo' => $self->entryinfo(),
			'entry'     => [],
			'entry_num' => 0,
		);
		$self->_generate_javascript($file,$temp);
	}
	return if ($type ne 'all' and $type ne 'category_list');
	# $type eq 'all' or $type eq 'category_list'
	foreach my $cat ( values %{$self->{'cat'}} )
	{
		next if ($cat->sub eq '');
		my $file = $dir . SUBCATEGORY_PREFIX . $cat->id . '.js';
		my $cms  = sb::Template->new(sprintf(LIST_TEMPLATE,'category','subcategory_list','category'));
		my $temp = $content->output($cms,
			'mode'      => 'cat',
			'id'        => $cat->id,
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
			'entryinfo' => $self->entryinfo(),
			'entry'     => [],
			'entry_num' => 0,
		);
		$self->_generate_javascript($file,$temp);
	}
}
sub build_cookie_js
{
	my $self  = shift;
	my $force = shift;
	my $file = $self->{'conf'}->value('conf_dir_base') . $self->{'conf'}->value('conf_dir_log') . $self->{'conf'}->value('file_cook');
	return if (-s $file and !$force);
	my $temp  = sb::Template->new(sb::App->load_template('file'=>COOKIE_JS_TEMP));
	my @icons = sb::Data->load('Image','cond'=>{'icon_c'=>1});
	my $icon_list = '';
	foreach my $icon (@icons)
	{
		$icon_list .= "\t" . 'case \'' . $icon->id . '\': imgSrc = "';
		$icon_list .= $icon->get_url . '"; break;' . "\n";
	}
	$temp->num(0);
	$temp->tag('sb_cookie_tag'=>$self->{'conf'}->value('basic_cooktag'));
	$temp->tag('sb_icon_list'=>$icon_list);
	eval {
		open(COOKIEJS,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(COOKIEJS);
		print COOKIEJS $temp->output;
		close(COOKIEJS);
		chmod($self->{'conf'}->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_feedfile
{
	my $self = shift;
	my $type = shift;
	my @feeds = keys( %mFeedFile );
	if (grep { /\Q$type\E/ } @feeds)
	{
		@feeds = ($type);
	}
	else
	{
		$type = 'all';
	}
	my $content = sb::Content->new('main' => { sb::Content::Feed::_init() });
	foreach my $feed (@feeds)
	{
		my $num = $self->{'conf'}->value('basic_gen_' . $feed);
		my $filebase = $self->{'conf'}->value('conf_dir_log') . $mFeedFile{$feed};
		my $file = $self->{'conf'}->value('conf_dir_base') . $filebase;
		my $temp = sb::App->load_template('file'=>sb::App::Feed->template_name($feed));
		sb::Content::FeedParam->new->path($self->{'conf'}->value('conf_srv_base') . $filebase);
		$self->set_latest_entries($num) if ($num != $self->{'conf'}->value('conf_entry_disp'));
		next if ($temp eq '');
		next if ($num == 0);
		my $body = $content->output(sb::Template->new($temp),
			'mode'      => 'page',
			'time'      => $self->{'time'},
			'blog'      => $self->{'blog'},
			'user'      => $self->{'user'},
			'cat'       => $self->{'cat'},
			'sortedcat' => $self->{'sortedcat'},
			'entryinfo' => $self->entryinfo(),
			'entry'     => $self->{'entry'},
			'entry_num' => $self->{'entry_num'},
			'num'       => $num,
		);
		if ($self->{'lang'}->charcode ne sb::App::Feed->get_code())
		{ # convert character code
			$body = $self->{'lang'}->convert($body,sb::App::Feed->get_code(),$self->{'lang'}->charcode);
		}
		eval {
			open(TOPOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
			binmode(TOPOUT);
			print TOPOUT $body;
			close(TOPOUT);
			chmod($self->{'conf'}->value('basic_file_attr'),$file);
		};
		last if ($@);
	}
	return $@ if ($@);
	return;
}
# --------------------------------------
# // public functions - accessors
sub template
{
	my $self = shift;
	return $self->{'template'} if ($self->{'template'});
	my %template_of = (
		'css'  => undef,
		'html' => {},
	);
	my @templates = sb::Data->load('Template');
	foreach my $temp ( @templates )
	{
		next if ($temp->main eq '' and $temp->entry eq '');
		if ($temp->use)
		{
			$self->{'default_id'} = $temp->id;
			$template_of{'css'} = sb::Template->new($temp->css) if ($temp->css ne '');
		}
		$template_of{'html'}->{$temp->id} = {
			'main'  => ($temp->main ne '')  ? sb::Template->new($temp->main)  : undef,
			'entry' => ($temp->entry ne '') ? sb::Template->new($temp->entry) : undef,
		};
	}
	die(sb->string('Unknown error.')) if ($self->{'default_id'} eq '');
	$self->{'template'} = \%template_of;
	return $self->{'template'};
}
sub entryinfo
{
	my $self = shift;
	return $self->{'entryinfo'} if ($self->{'entryinfo'});
	$self->{'entryinfo'} = sb::Content::_entry_info(
		'time' => $self->{'time'},
		'mode' => 'page',
		'conf' => $self->{'conf'},
	);
	return $self->{'entryinfo'};
}
sub set_latest_entries
{
	my $self = shift;
	my $num  = shift;
	$num = $self->{'conf'}->value('conf_entry_disp') if (!$num);
	$self->{'entry'} = [sb::Data->load('Entry',
			'num'   => $num,
			'cond'  => {'stat'=>1},
			'sort'  => 'date',
			'order' => 1,
		)
	];
	$self->{'entry_num'} = sb::Data->matched;
}
sub register_js_list
{
	my $self = shift;
	my $name = shift;
	my %param = (
		'file'  => '',
		'block' => '',
		'entry' => undef,
		@_
	);
	return if ( !$name or !$param{'file'} or !$param{'block'} );
	$self->{'js_list'}->{$name}->[JS_FILE]  = $param{'file'};
	$self->{'js_list'}->{$name}->[JS_BLOCK] = $param{'block'};
	$self->{'js_list'}->{$name}->[JS_ENTRY] = $param{'entry'};
}
sub run_callback
{
	my $self = shift;
	my $type = shift;
	my $option = shift;
	my @funcs = @{$self->{'callback'}};
	return if ( !@funcs );
	foreach my $func (@funcs)
	{
		eval{ &$func($self,$type,$option); };
	}
}
# --------------------------------------
# // private functions - utilities
sub _check_template
{
	my $self = shift;
	my $mode = shift; # mode 'ent','arc', or none
	my $cid  = shift; # category id
	my ($base,$css); # output parameters
	my $temp = undef;
	my $cat = ($cid ne '') ? $self->{'cat'}->{$cid} : undef;
	my $tid = ($mode eq 'arc') ? $self->{'conf'}->value('conf_archive_temp') : -1;
	$tid = $cat->temp if ($cat and $cat->temp ne '' and $cat->temp > -1);
	$tid = undef if ($tid == -1);
	$css = ( $tid ne '' and $self->{'conf'}->value('conf_css_change') ) # css path
	     ? $self->{'conf'}->value('conf_srv_cgi') . $self->{'conf'}->value('basic_sb') . '?css=' . $tid 
	     : $self->{'conf'}->value('conf_srv_base') . $self->{'conf'}->value('file_css');
	$temp = $self->template()->{'html'}->{$tid} if ($tid ne '');
	$temp = $self->template()->{'html'}->{$self->{'default_id'}} if (!$temp);
	$base = ($mode eq 'ent' and $temp->{'entry'}) # template base
	      ? $temp->{'entry'} 
	      : $temp->{'main'}; 
	$base->clear();
	return($base,$css);
}
sub _generate_javascript
{
	my $self = shift;
	my $file = shift;
	my $body = shift;
	eval {
		open(JSOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(JSOUT);
		foreach my $line ( split("\n",$body) )
		{
			$line =~ s/\'/&#39;/g;
			print JSOUT 'document.write(\'' . $line . '\\n\');',"\n";
		}
		close(JSOUT);
		chmod($self->{'conf'}->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
# --------------------------------------
# // private functions - content
sub _set_script_tags
{
	my $self = shift;
	my $mode = shift;
	my %callback_of = ();
	foreach my $tag ( keys %{$self->{'js_list'}} )
	{
		my $block = $self->{'js_list'}->{$tag}->[JS_BLOCK];
		next if ( $block eq 'category' or $block eq 'calendar' ); # exception
		$callback_of{$block} = sub { $self->_script_tag($tag,$_[0]); };
	}
	$callback_of{'category'} = sub { $self->_category_list(@_); } if ( $mode ne 'category' );
	$callback_of{'calendar'} = sub { $self->_calendar_for_entry(@_); } if ( $mode ne 'monthly' );
	return %callback_of;
}
sub _script_tag
{
	my $self = shift;
	my $key  = shift;
	my $cms  = shift;
	my $conf = sb::Config->get;
	my $file = $conf->value('conf_dir_log') . $self->{'js_list'}->{$key}->[JS_FILE];
	return 0 if (!-e $conf->value('conf_dir_base') . $file);
	return 0 if (-z $conf->value('conf_dir_base') . $file);
	$cms->num(0);
	$cms->tag($key => sprintf(SCRIPT_TAG,sb::Language->get->charset,$conf->value('conf_srv_base') . $file));
	return 1;
}
sub _calendar_for_entry
{
	my $self = shift;
	$self->_script_tag('calendar',$_[0]);
	$self->_script_tag('calendar2',$_[0]);
	$self->_script_tag('calendar_vertical',$_[0]);
	$self->_script_tag('calendar_horizontal',$_[0]);
	return 1;
}
sub _category_list
{
	my $self = shift;
	my $cms = shift;
	my %var = @_;
	my $check = $self->_script_tag('category_list',$cms);
	if ($check and $var{'entry'}->[0] and $var{'entry'}->[0]->cat ne '')
	{
		my $cat = $var{'cat'}->{$var{'entry'}->[0]->cat};
		if ($cat and $cat->sub ne '')
		{
			my $path = $var{'conf'}->value('conf_srv_base') . $var{'conf'}->value('conf_dir_log');
			$path .= SUBCATEGORY_PREFIX . $cat->id . '.js';
			$cms->num(0);
			$cms->tag('subcategory_list'=>sprintf(SCRIPT_TAG,$var{'lang'}->charset,$path));
		}
		return 1;
	}
	return $check;
}
sub _category_feeds
{
	my $cms = shift;
	my %var = @_;
	my $path = $var{'conf'}->value('conf_srv_cgi') . $var{'conf'}->value('basic_feed');
	$cms->tag('site_rss' =>$path . 'rss&amp;cid=' . $var{'id'});
	$cms->tag('site_atom'=>$path . 'atom&amp;cid=' . $var{'id'});
}
1;
__END__
