/*!
  spica javascript libraries - buildprogress.js
  == written by Takuya Otani <takuya.otani@gmail.com> ===
  == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==
*/

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.02 [2009/04/15] renamed to run to consider upgrading
// ver 0.01 [2009/01/18] added elapsed as a member variable
// ver 0.00 [2008/11/01] the first attempt

Spica.Library.require('ajax','element','benchmark');
Spica.BuildProgress = function(serverUrl)
{
	this.initialized = false;
	this.started = false;
	this.timer = null;
	this.num = 0;
	this.max = 0;
	this.debug = 0;
	this.query = '__rebuild';
	this.elapsed = null;
	this.server = serverUrl;
	return this;
};
Spica.BuildProgress.prototype = {
	doTest : function()
	{
		var self = this;
		if (self.initialized) return;
		var ajax = new Spica.Ajax();
		if (!ajax)
		{ // just set a flag
			self.initialized = true;
			return;
		}
		ajax.onerror  = function() {}; // do nothing
		ajax.callback = function(response) { self.doneTest(response) };
		ajax.get(self.server + '?' + self.query + '=-1',null);
	},
	doneTest : function(response)
	{
		var self = this;
		self.initialized = true;
		if (!response) return;
		var res = response.getElementsByTagName('result');
		if (!res.length || !res[0].firstChild.nodeValue || res[0].firstChild.nodeValue != 1) return;
		if ( self.query == '__rebuild' )
		{
			Spica.$('rebuild_option').options.length = 2;
			Spica.Event.register(Spica.$('submit'),'click',function(evt) { self.onSubmit(evt) });
		}
		else
		{
			var checkObj = document.getElementById('check_xml');
			if (checkObj) checkObj.value = 'on';
		}
	},
	onSubmit : function(evnt)
	{
		var self = this;
		evnt = (evnt) ? evnt : ((window.event) ? event : null);
		if (Spica.$('rebuild_option').options[0].selected)
		{
			self.run();
			return Spica.Event.stop(evnt);
		}
	},
	run : function()
	{
		var self = this;
		if (self.max == 0) return false; // do nothing
		if (!self.started)
		{
			self.elapsed = new Spica.Benchmark();
			if ( self.query == '__rebuild' )
			{
				Spica.$('rebuildOptionField').style.display = 'none';
				Spica.$('rebuildStatusField').style.display = 'block';
			}
			else
			{
				Spica.$('formStep3').style.display = 'none';
			}
			Spica.$('explainComplete').style.display = 'none';
			Spica.$('explainProgress').style.display = 'block';
			self.started = true;
		}
		var ajax = new Spica.Ajax();
		if (!ajax) return false;
		ajax.onerror  = function() {}; // do nothing
		ajax.callback = function(response) { self.progress(response) };
		ajax.get(self.server + '?' + self.query + '=' + self.num,null);
	},
	progress : function(response)
	{
		var self = this;
		if (!response) return self.showMessage(Spica.$_('No content error.'));
		var result = response.getElementsByTagName('result');
		if (!result.length) return self.showMessage(Spica.$_('No result error.'));
		var check = result[0].firstChild.nodeValue;
		if (self.num > self.max) return self.showMessage(Spica.$_('Unknown error.'));
		if (check == 1)
		{
			self.showMessage('');
			self.num++;
			if (self.timer) window.clearTimeout(self.timer);
			self.timer = window.setTimeout(function() { self.run(); }, 100);
		}
		else
		{ // check != 1
			var extra_msg = ( self.debug > 0 ) ? self.elapsed.toString() : '';
			self.showMessage('<strong>' + Spica.$_(check) + '</strong> ' + extra_msg);
			if (check == 'completed')
			{
				if ( self.query != '__rebuild' )
				{
					Spica.$('formStep3').style.display = 'block';
				}
				Spica.$('explainComplete').style.display = 'block';
				Spica.$('explainProgress').style.display = 'none';
			}
		}
	},
	showMessage : function(message)
	{
		var self = this;
		var progress = Spica.$('rebuildProgress');
		if (progress && Spica.DOM.getParent(progress))
		{
			var width = Spica.DOM.getParent(progress).clientWidth;
			if (message == '') width *= (self.num / self.max);
			progress.style.width = Math.floor(width) + 'px';
		}
		message = Math.floor((self.num / self.max) * 100) + ' % ' + message;
		if (!progress)
			alert(message);
		else
			progress.innerHTML = message;
		return;
	}
};
