/*!
  spica javascript libraries - template.js
  == written by Takuya Otani <takuya.otani@gmail.com> ===
  == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==
*/

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.01 [2009/02/24] fixed a bug in _output_block
// ver 0.00 [2009/02/20] generated

Spica.Template = function(template,callback)
{
	this.number = 0;
	this.data = { tag: {}, block: {} };
	this.blocks = this._parse( template.split("\n"), callback );
	return this;
}
Spica.Template.prototype = {
	output : function()
	{
		this.data.block['-main'] = 1;
		return this._output_block('-main');
	},
	deleteBlock : function( name )
	{
		if ( !block ) return;
		if ( this.blocks[name] ) this.blocks[name].text = '';
	},
	clear : function()
	{
		this.data.tag = {};
		for (var name in this.data.block)
			this.data.block[name] = 0;
	},
	tag : function( tag, content )
	{
		if ( !tag ) return;
		if ( !this.data.tag['{' + tag + '}'] )
			this.data.tag['{' + tag + '}'] = [];
		this.data.tag['{' + tag + '}'][ this.number ] = content;
	},
	num : function( num )
	{
		this.number = num;
	},
	block : function( name, num )
	{
		if ( !name ) return;
		this.data.block[ name ] = num;
	},
	existed : function( name )
	{
		if ( name && this.blocks[name] ) return true;
		return false;
	},
	_output_block : function( name )
	{
		if ( !name ) return '';
		if ( !this.blocks[name] ) return '';
		if ( !this.data.block[name] ) return '';
		if ( this.blocks[name].text == '' ) return '';
		var output = '';
		for (var i=0,n=this.data.block[name];i<n;i++)
		{
			var text = this.blocks[name].text;
			for (var j=0,m=this.blocks[name].tags.length;j<m;j++)
			{
				var check = new String(this.blocks[name].tags[j]);
				var result = 
				  check.match(/\{\-(\w+)\}/)                     ? this._output_block(RegExp.$1)
				: typeof(this.data.tag[check]) == 'undefined'    ? ''
				: typeof(this.data.tag[check][i]) != 'undefined' ? this.data.tag[check][i]
				: typeof(this.data.tag[check][0]) != 'undefined' ? this.data.tag[check][0]
				:                                                  '';
				text = text.replace(new RegExp(check,'g'),result);
			}
			output += text;
		}
		return output;
	},
	_parse : function( lines, callback )
	{
		var block = {};
		var stack = [];
		var cur = '-main';
		if ( !callback ) callback = this._default_callback;
		if ( typeof(callback) != 'function' ) callback = function() {};
		block[cur] = { text:'', tags:[] };
		stack.push(cur);
		for (var i=0,n=lines.length;i<n;i++)
		{
			var line = lines[i];
			if ( line.match(/<\!-- BEGIN (\w+) -->/) )
			{
				block[cur].text += '{-' + RegExp.$1 + '}';
				block[cur].tags.push('{-' + RegExp.$1 + '}');
				cur = RegExp.$1;
				block[cur] = { text:'', tags:[] };
				block = callback(cur,false,block);
				stack.push(cur);
				continue;
			}
			else if ( line.indexOf('<!-- END ') > -1 )
			{
				block = callback(cur,true,block);
				var tmp = block[cur].tags;
				var m = tmp.length;
				block[cur].tags = [];
				while ( m ) if ( block[cur].tags.indexOf(tmp[--m]) == -1 ) block[cur].tags.push(tmp[m]);
				stack.pop();
				cur = stack[ stack.length - 1 ];
				continue;
			}
			if ( line.match(/\{\w+?\}/) )
			{
				block[cur].tags.push( line.match(/(\{\w+?\})/g) );
			}
			block[cur].text += line + "\n";
		}
		return block;
	},
	_default_callback : function(name,isEnd,block)
	{
		return block;
	}
};
